#!/usr/bin/env python3
#
from __future__ import annotations
from enum import Enum
from convert_stream import ArrayString
import os.path
import pandas as pd


def contains(text: str, values: list[str], *, case: bool = True, iqual: bool = False) -> bool:
    if case:
        if iqual:
            for x in values:
                if text == x:
                    return True
        else:
            for x in values:
                if text in x:
                    return True
    else:
        if iqual:
            for x in values:
                if text.upper() == x.upper():
                    return True
        else:
            for x in values:
                if text.upper() in x.upper():
                    return True
    return False


class HeadItem(str):

    def __init__(self, text: str):
        super().__init__()
        self.text: str = text


class HeadValues(list):

    def __init__(self, head_items: list[HeadItem]):
        super().__init__(head_items)

    def contains(self, item: HeadItem | str, *, case: bool = True, iqual: bool = False) -> bool:
        return contains(item, self, case=case, iqual=iqual)

    def get(self, idx: int) -> HeadItem:
        return self[idx]

    def add_head(self, h: HeadItem):
        if isinstance(h, HeadItem):
            self.append(h)


class ColumnBody(list):

    def __init__(self, col_name: HeadItem | str, col_body: list[str] = []):
        super().__init__(col_body)
        if isinstance(col_name, str):
            col_name = HeadItem(col_name)
        self.col_name: HeadItem = col_name

    @property
    def length(self) -> int:
        return len(self)

    @property
    def is_empty(self) -> bool:
        return self.length == 0

    def __repr__(self):
        return f'{self.col_name}: {super().__repr__()}'

    def add_line(self, line: str):
        if isinstance(line, str):
            self.append(line)

    def add_lines(self, lines: list[str]):
        for line in lines:
            self.add_line(line)


class ColumnNames(Enum):

    NUM_LINE = 'LINHA'
    NUM_PAGE = 'PÁGINA'
    TEXT = 'TEXTO'
    FILE_PATH = 'ARQUIVO'
    FILE_NAME = 'NOME_ARQUIVO'


class TextTable(object):

    def __init__(self, items: list[ColumnBody] = []):
        self.items: list[ColumnBody] = items

    def __repr__(self):
        return f'{self.__class__.__name__}:\n{self.items})'

    @property
    def head(self) -> HeadValues:
        return HeadValues([x.col_name for x in self.items])

    @property
    def length(self) -> int:
        return len(self.items[0])

    @property
    def is_empty(self) -> bool:
        return self.length == 0

    def add_column(self, col: ColumnBody, *, replace: bool = False):
        if isinstance(col, ColumnBody):
            if self.head.contains(col.col_name, case=True, iqual=True):
                if replace:
                    idx_col = self.head.index(col.col_name)
                    self.items[idx_col] = col
            else:
                self.items.append(col)

    def extend_column(self, col: ColumnBody):
        if not self.head.contains(col.col_name, case=True, iqual=True):
            raise IndexError(f'Column {col.col_name} não existe em: {self.head}')
        idx = self.head.index(col.col_name)
        self.items[idx].extend(col)

    def to_array(self) -> ArrayString:
        return ArrayString(self.get_column(ColumnNames.TEXT.value))

    def to_dict(self) -> dict[HeadItem, ColumnBody]:
        v = {}
        for col in self.items:
            v[col.col_name] = col
        return v

    def to_data(self) -> pd.DataFrame:
        return pd.DataFrame.from_dict(self.to_dict())

    def get_column(self, name: str) -> ColumnBody | None:
        for col in self.items:
            if col.col_name == name:
                return col
        return None


def create_map_text(values: list[str], filepath: str, *, page_num: str = 'nan') -> TextTable:
    total = len(values)
    col_num_page = ColumnBody(ColumnNames.NUM_PAGE.value, [page_num] * total)
    col_num_lines = ColumnBody(ColumnNames.NUM_LINE.value, [f'{x+1}' for x in range(total)])
    colum_txt = ColumnBody(ColumnNames.TEXT.value, values)
    colum_filepath = ColumnBody(ColumnNames.FILE_PATH.value, [filepath] * total)
    colum_filename = ColumnBody(ColumnNames.FILE_NAME.value, [os.path.basename(filepath)] * total)
    return TextTable([col_num_page, col_num_lines, colum_txt, colum_filepath, colum_filename])







#!/usr/bin/env python3

from __future__ import annotations
import os.path
import pandas as pd
from soup_files import File, Directory, JsonConvert, ProgressBarAdapter
from organize.read import ColumnNames, HeadItem, HeadValues, ColumnBody, TextTable
from convert_stream import ArrayString


class SearchableText(object):

    default_elements: dict[HeadItem, ColumnBody] = {
        HeadItem(ColumnNames.NUM_PAGE.value): ColumnBody(ColumnNames.NUM_PAGE.value),
        HeadItem(ColumnNames.NUM_LINE.value): ColumnBody(ColumnNames.NUM_LINE.value),
        HeadItem(ColumnNames.TEXT.value): ColumnBody(ColumnNames.TEXT.value),
        HeadItem(ColumnNames.FILE_PATH.value): ColumnBody(ColumnNames.FILE_NAME.value),
        HeadItem(ColumnNames.FILE_NAME.value): ColumnBody(ColumnNames.FILE_NAME.value),
    }

    default_columns: HeadValues = HeadValues(
        [
            HeadItem(ColumnNames.NUM_PAGE.value),
            HeadItem(ColumnNames.NUM_LINE.value),
            HeadItem(ColumnNames.TEXT.value),
            HeadItem(ColumnNames.FILE_PATH.value),
            HeadItem(ColumnNames.FILE_NAME.value),
        ]
    )

    def __init__(self):
        
        self.elements: dict[HeadItem, ColumnBody] = {
            HeadItem(ColumnNames.NUM_PAGE.value): ColumnBody(ColumnNames.NUM_PAGE.value),
            HeadItem(ColumnNames.NUM_LINE.value): ColumnBody(ColumnNames.NUM_LINE.value),
            HeadItem(ColumnNames.TEXT.value): ColumnBody(ColumnNames.TEXT.value),
            HeadItem(ColumnNames.FILE_PATH.value): ColumnBody(ColumnNames.FILE_NAME.value),
            HeadItem(ColumnNames.FILE_NAME.value): ColumnBody(ColumnNames.FILE_NAME.value),
        }

    def __repr__(self):
        return f'SearchableText\nHead: {self.head}\nBody: {self.body}'

    def is_empty(self) -> bool:
        return len(self.elements[HeadItem(ColumnNames.TEXT.value)]) == 0

    @property
    def head(self) -> HeadValues:
        return HeadValues([HeadItem(x) for x in list(self.elements.keys())])

    @property
    def body(self) -> list[ColumnBody]:
        return [ColumnBody(HeadItem(_k), self.elements[_k]) for _k in self.elements.keys()]

    @property
    def first(self) -> dict[str, str]:
        if self.is_empty():
            return {}
        cols: HeadValues = self.head
        _first = {}
        for col in cols:
            _first[col] = self.elements[col][0]
        return _first

    @property
    def last(self) -> dict[str, str]:
        if self.is_empty():
            return {}
        cols = self.head
        _last = {}
        for col in cols:
            _last[col] = self.elements[col][-1]
        return _last

    @property
    def length(self) -> int:
        return len(self.elements[HeadItem(ColumnNames.TEXT.value)])

    @property
    def files(self) -> ColumnBody:
        return self.elements[HeadItem(ColumnNames.FILE_PATH.value)]

    def get_item(self, idx: int) -> dict[str, str]:
        cols: HeadValues = self.head
        try:
            _item = {}
            for col in cols:
                _item[col] = self.elements[col][idx]
            return _item
        except Exception as err:
            print(err)
            return {}

    def get_column(self, name: str) -> ColumnBody:
        return self.elements[HeadItem(name)]

    def add_line(self, text: str, *, num_line: str, file: str, num_page: str) -> None:
        self.elements[HeadItem(ColumnNames.NUM_PAGE.value)].add_line(num_page)
        self.elements[HeadItem(ColumnNames.NUM_LINE.value)].append(num_line)
        self.elements[HeadItem(ColumnNames.TEXT.value)].append(text)
        self.elements[HeadItem(ColumnNames.FILE_PATH.value)].append(file)
        self.elements[HeadItem(ColumnNames.FILE_NAME.value)].append(os.path.basename(file))

    def clear(self) -> None:
        for _k in self.elements.keys():
            self.elements[_k].clear()

    def to_string(self) -> str:
        """
            Retorna o texto da coluna TEXT em formato de string
        ou 'nas' em caso de erro nas = Not a String
        """
        try:
            return ' '.join(self.elements[HeadItem(ColumnNames.TEXT.value)])
        except Exception as e:
            print(e)
            return 'nan'

    def to_data_frame(self) -> pd.DataFrame:
        return pd.DataFrame.from_dict(self.elements)

    def to_file_json(self, file: File):
        """Exporta os dados da busca para arquivo .JSON"""
        dt = JsonConvert.from_dict(self.elements).to_json_data()
        dt.to_file(file)

    def to_file_excel(self, file: File):
        """Exporta os dados da busca para arquivo .XLSX"""
        self.to_data_frame().to_excel(file.absolute(), index=False)

    @classmethod
    def create(cls, df: pd.DataFrame) -> SearchableText:
        default = cls.default_elements
        default[HeadItem(ColumnNames.NUM_PAGE.value)] = ColumnBody(
            ColumnNames.NUM_PAGE.value, df[ColumnNames.NUM_PAGE.value].values.tolist(),
        )
        default[HeadItem(ColumnNames.NUM_LINE.value)] = ColumnBody(
            ColumnNames.NUM_LINE.value, df[ColumnNames.NUM_LINE.value].values.tolist()
        )
        default[HeadItem(ColumnNames.TEXT.value)] = ColumnBody(
            ColumnNames.TEXT.value, df[ColumnNames.TEXT.value].values.tolist()
        )
        default[HeadItem(ColumnNames.FILE_PATH.value)] = ColumnBody(
            ColumnNames.FILE_PATH.value, df[ColumnNames.FILE_PATH.value].values.tolist()
        )
        default[HeadItem(ColumnNames.FILE_NAME.value)] = ColumnBody(
            ColumnNames.FILE_NAME.value, df[ColumnNames.FILE_NAME.value].values.tolist()
        )
        df_fmt = df.drop(columns=cls.default_columns).astype('str')
        new_columns: list[str] = df_fmt.columns.tolist()

        for col in new_columns:
            default[HeadItem(col)] = ColumnBody(col, df_fmt[col].values.tolist())
        _s = cls()
        _s.elements = default
        return _s


class DocumentFinder(object):

    def __init__(self, text_maps: list[TextTable]):
        self.text_maps: list[TextTable] = text_maps

        # Coluna que contem o texto usado como filtro em cada linha da busca
        self._col_name_filter: str = 'FILTRO'
        # Coluna que contem o texto de filtro adicional
        self._col_include_filter: str = 'FILTRO ADICIONAL'

    def add_table(self, item: TextTable) -> None:
        self.text_maps.append(item)

    def find(self, text: str) -> SearchableText:
        _data = []
        s = SearchableText()
        for text_map in self.text_maps:
            arr = text_map.to_array()
            _current_idx = arr.index(text)
            if _current_idx is not None:
                text_dict: dict[HeadItem | str, ColumnBody] = text_map.to_dict()
                s.add_line(
                    text=text_dict[ColumnNames.TEXT.value][_current_idx],
                    num_line=text_dict[ColumnNames.NUM_LINE.value][_current_idx],
                    file=text_dict[ColumnNames.FILE_PATH.value][_current_idx],
                    num_page=text_dict[ColumnNames.NUM_PAGE.value][_current_idx],
                )
                break
        return s

    def find_all(self, text: str) -> SearchableText:
        _data = []
        s = SearchableText()
        for tb in self.text_maps:
            arr: ArrayString = tb.to_array()
            _current_idx: int = arr.index(text)
            if _current_idx is not None:
                s.add_line(
                    text=tb.get_column(ColumnNames.TEXT.value)[_current_idx],
                    num_line=tb.get_column(ColumnNames.NUM_LINE.value)[_current_idx],
                    file=tb.get_column(ColumnNames.FILE_PATH.value)[_current_idx],
                    num_page=tb.get_column(ColumnNames.NUM_PAGE.value)[_current_idx],
                )
        return s

    def find_from_data(self, df: pd.DataFrame, *, col_filter: str, col_new_filter: str = None) -> SearchableText:
        """
            Filtrar vários textos em documentos/arquivos, incluindo dois tipos de filtros simultaneos.

        :df: DataFrame com os textos a serem usados como filtro apartir da coluna indicada
        :col_filter: string da coluna que contém os textos a serem filtrados
        :col_new_filter: string da coluna que contém um filtro adicional (opcional)

        @type df: pd.DataFrame
        @type col_filter: str
        @type col_new_filter: str
        @rtype: SearchableText
        """
        if col_new_filter is not None:
            df = df[[col_filter, col_new_filter]].dropna(subset=[col_filter]).astype('str')
        else:
            df = df[[col_filter]].dropna(subset=[col_filter]).astype('str')

        list_data = []
        for i, row in df.iterrows():
            _searchable = self.find_all(f'{row[col_filter]}').to_data_frame()
            if not _searchable.empty:
                if col_new_filter is not None:
                    new_searchable = self.find_all(f'{row[col_new_filter]}').to_data_frame()
                    if not new_searchable.empty:
                        _searchable = pd.concat([_searchable, new_searchable])
                        _searchable[self._col_name_filter] = [row[col_filter]] * len(_searchable)
                        _searchable[self._col_include_filter] = [row[col_new_filter]] * len(_searchable)
                    else:
                        _searchable[self._col_name_filter] = [row[col_filter]] * len(_searchable)
                        _searchable[self._col_include_filter] = ['nan'] * len(_searchable)
                else:
                    _searchable[self._col_name_filter] = [row[col_filter]] * len(_searchable)
                    _searchable[self._col_include_filter] = ['nan'] * len(_searchable)
                list_data.append(_searchable)

        if len(list_data) > 0:
            return SearchableText.create(pd.concat(list_data))
        return SearchableText()




#!/usr/bin/env python3

__version__ = '1.0'
from .read import (
    ImageObject, TextTable, contains, create_map_text, merge_text_table,
    ColumnNames, ColumnBody, HeadItem, HeadValues
)
from .find import SearchableText, DocumentFinder
from .document import OrganizeDocuments, DocumentTextExtract




#!/usr/bin/env python3

from __future__ import annotations
from io import BytesIO
from typing import Union

import pandas as pd
from convert_stream.image.img_object import ABCImageObject

from .text_map import (
    ColumnNames, HeadItem, HeadValues, ArrayString, ColumnBody, TextTable, contains, create_map_text
)
from convert_stream import (
    DocumentPdf, PageDocumentPdf, ConvertPdfToImages, ImageObject as ImObj, LibImage, DEFAULT_LIB_IMAGE
)
from ocr_stream import RecognizeImage, RecognizePdf, BinTesseract, TextRecognized, LibOcr, DEFAULT_LIB_OCR
from soup_files import Directory, File, InputFiles, LibraryDocs, ProgressBarAdapter


class ImageObject(ImObj):

    def __init__(
                self, img: Union[ABCImageObject, bytes, BytesIO, str, File], *,
                lib_image: LibImage = DEFAULT_LIB_IMAGE
            ) -> None:
        if isinstance(img, File):
            self.name_extension: str = img.extension()
        else:
            self.name_extension = None
        super().__init__(img, lib_image=lib_image)


def merge_text_table(text_list: list[TextTable]) -> pd.DataFrame:
    if len(text_list) == 0:
        return pd.DataFrame()
    list_data: list[pd.DataFrame] = []
    for tb in text_list:
        df = tb.to_data()
        list_data.append(df)
    final_df = pd.concat(list_data)
    return final_df


class Ocr(RecognizeImage):
    _instance = None  # armazena a instância única

    def __new__(cls, bin_tess: BinTesseract = BinTesseract(), *, lib_ocr: LibOcr = DEFAULT_LIB_OCR):
        if cls._instance is None:
            # Cria a instância uma única vez
            cls._instance = super(Ocr, cls).__new__(cls)
        return cls._instance

    def __init__(self, bin_tess: BinTesseract = BinTesseract(), *, lib_ocr: LibOcr = DEFAULT_LIB_OCR):
        # Evita reexecutar __init__ em chamadas subsequentes
        if not hasattr(self, "_initialized"):
            super().__init__(bin_tess, lib_ocr=lib_ocr)
            self._initialized = True


def recognize_images(
        images: list[ImageObject], *,
        ocr: Ocr = Ocr(),
        pbar: ProgressBarAdapter = ProgressBarAdapter()
) -> DocumentPdf:
    """
        Aplicar OCR em lista de imagens, e retornar um documento DocumentPdf() com as imagens embutidas.
    """
    pages_pdf: list[PageDocumentPdf] = []
    max_num: int = len(images)
    print()
    pbar.start()
    for _num, im in enumerate(images):
        pbar.update(
            ((_num + 1) / max_num) * 100,
            f'[OCR]: {_num + 1}/{max_num} {im.name}',
        )
        tmp_doc = ocr.image_recognize(im).to_document()
        pages_pdf.extend(tmp_doc.to_pages())
        del tmp_doc
    pbar.stop()
    print()
    return DocumentPdf.create_from_pages(pages_pdf)


def read_image(
        image: File | ImageObject | bytes | BytesIO, *,
        file_path: str = None,
        ocr: Ocr = Ocr(),
) -> TextTable:
    if isinstance(image, File):
        if file_path is None:
            file_path = image.absolute()
    elif isinstance(image, ImageObject):
        pass
    elif isinstance(image, bytes):
        image = ImageObject.create_from_bytes(image)
    elif isinstance(image, BytesIO):
        image = ImageObject(image)
    else:
        raise ValueError('Use: File|DocumentPdf|bytes|ByesIO')

    if file_path is None:
        file_path = image.name
        if image.name_extension is not None:
            file_path = f'{file_path}{image.name_extension}'
    try:
        text_image: str = ocr.image_to_string(image)
        if (text_image is None) or (text_image == ''):
            mp = create_map_text(['nan'], file_path)
        else:
            mp = create_map_text(text_image.split('\n'), file_path)
    except Exception as err:
        print(err)
        mp = create_map_text(['nan'], file_path)
    return mp


def read_files_image(
        images_files: list[ImageObject], *,
        ocr: Ocr = Ocr(),
        pbar: ProgressBarAdapter = ProgressBarAdapter(),
) -> list[TextTable]:
    """
        Ler as imagens de um diretório e retorna list[TextMap]
    """

    list_table: list[TextTable] = []
    max_num: int = len(images_files)
    print()
    pbar.start()
    for _num, img in enumerate(images_files):
        pbar.update(
            ((_num + 1) / max_num) * 100,
            f'[OCR]: {_num + 1}/{max_num} {img.name}',
        )
        list_table.append(read_image(img, ocr=ocr))
    return list_table


def read_directory_image(
        directory: Directory, *,
        ocr: Ocr = Ocr(),
        pbar: ProgressBarAdapter = ProgressBarAdapter(),
) -> list[TextTable]:
    """
        Ler as imagens de um diretório e retorna list[TextMap]
    """
    images_files: list[File] = InputFiles(directory).get_files(file_type=LibraryDocs.IMAGE)
    _data: list[TextTable] = []
    max_num: int = len(images_files)
    print()
    pbar.start()
    for _num, file_image in enumerate(images_files):
        pbar.update(
            ((_num + 1) / max_num) * 100,
            f'[OCR]: {_num + 1}/{max_num} {file_image.basename()}',
        )
        im = ImageObject(file_image)
        _data.append(read_image(im, ocr=ocr, file_path=file_image.absolute()))
    return _data


def read_document_pdf(
        document: DocumentPdf,
        file_path: str,
        apply_ocr: bool = False,
        ocr: Ocr = Ocr(),
        pbar: ProgressBarAdapter = ProgressBarAdapter(),
) -> list[TextTable]:
    """Extrair os textos de páginas PDF e retornar um objeto TextMap()"""
    if not isinstance(document, DocumentPdf):
        raise TypeError(f'file_pdf dev ser DocumentPdf() não {type(file_pdf)}')
    if apply_ocr:
        conv = ConvertPdfToImages.create(document)
        images: list[ImageObject] = conv.to_images()
        document: DocumentPdf = recognize_images(images, ocr=ocr, pbar=pbar)
    list_table = []
    pages_pdf = document.to_pages()
    for pg in pages_pdf:
        text_page = pg.to_string()
        if (text_page is None) or (text_page == '') or (text_page == 'nas'):
            current_mp = create_map_text(['nan'], file_path, page_num=f'{pg.number_page}')
        else:
            try:
                current_mp = create_map_text(text_page.split('\n'), file_path, page_num=f'{pg.number_page}')
            except Exception as err:
                print(err)
                current_mp = create_map_text(['nan'], file_path, page_num=f'{pg.number_page}')
        list_table.append(current_mp)
    return list_table


def read_file_pdf(
        file_pdf: File,
        apply_ocr: bool = False,
        ocr: Ocr = Ocr(),
        pbar: ProgressBarAdapter = ProgressBarAdapter(),
) -> list[TextTable]:
    """Extrair os textos de páginas PDF e retornar um objeto TextMap()"""
    if not isinstance(file_pdf, File):
        raise TypeError(f'file_pdf dev ser File() não {type(file_pdf)}')
    return read_document_pdf(
        DocumentPdf(file_pdf), file_pdf.absolute(), ocr=ocr, pbar=pbar, apply_ocr=apply_ocr
    )


def read_directory_pdf(
        directory: Directory, *,
        apply_ocr: bool = False,
        ocr: Ocr = Ocr(),
        pbar: ProgressBarAdapter = ProgressBarAdapter(),
) -> list[TextTable]:
    files_doc_pdf: list[File] = InputFiles(directory).get_files(file_type=LibraryDocs.PDF)
    _text_maps: list[TextTable] = []
    for f_pdf in files_doc_pdf:
        _current_maps = read_file_pdf(f_pdf, apply_ocr=apply_ocr, ocr=ocr, pbar=pbar)
        _text_maps.extend(_current_maps)
    return _text_maps



#!/usr/bin/env python3
from __future__ import annotations
from typing import Union
from organize.find import SearchableText, DocumentFinder
from organize.read import (
    TextTable, ColumnBody, ColumnNames, read_directory_pdf, read_directory_image,
    read_file_pdf, read_document_pdf, read_image, read_files_image, merge_text_table, contains
)
from soup_files import File, Directory, ProgressBarAdapter
from convert_stream import DocumentPdf, ArrayString, ImageObject
import pandas as pd
import shutil

FindItem = Union[str, list[str]]

_bad_chars: list[str] = [
    '.', '!', ':', '?', '(', ')', '{', '}',
    '+', '#', '@', '<', '>', '/', '¢', ':',
]


def fmt_str_file(filename: str) -> str:
    for c in _bad_chars:
        filename = filename.replace(c, '')
    while '--' in filename:
        filename = filename.replace('--', '-')
    return filename


def move_list_files(mv_items: dict[str, list[File]], *, replace: bool = False) -> None:
    total_file = len(mv_items['src'])
    for idx, file in enumerate(mv_items['src']):
        if not file.exists():
            print(f'[PULANDO]: {idx + 1} Arquivo não encontrado {file.absolute()}')
        if mv_items['dest'][idx].exists():
            if not replace:
                print(f'[PULANDO]: {idx+1} O arquivo já existe {mv_items["dest"][idx].absolute()}')
                continue
        print(f'Movendo: {idx+1}/{total_file} {file.absolute()}')
        try:
            shutil.move(file.absolute(), mv_items['dest'][idx].absolute())
        except Exception as e:
            print(f'{e}')


class DocumentTextExtract(object):
    """
        Extrair texto de arquivos, e converter em Excel/DataFrame
    """

    def __init__(self):
        self.tb_list: list[TextTable] = []
        self.pbar: ProgressBarAdapter = ProgressBarAdapter()

    @property
    def is_empty(self) -> bool:
        return len(self.tb_list) == 0

    @property
    def finder(self) -> DocumentFinder:
        return DocumentFinder(self.tb_list)

    def add_directory_pdf(self, dir_pdf: Directory, *, apply_ocr: bool = False):
        self.tb_list.extend(read_directory_pdf(dir_pdf, apply_ocr=apply_ocr, pbar=self.pbar))

    def add_directory_image(self, dir_image: Directory):
        self.tb_list.extend(read_directory_image(dir_image, pbar=self.pbar))

    def add_file_pdf(self, file_pdf: File, apply_ocr: bool = False):
        self.tb_list.extend(read_file_pdf(file_pdf, pbar=self.pbar, apply_ocr=apply_ocr))

    def add_file_image(self, file_image: File):
        self.tb_list.append(read_image(file_image))

    def add_image(self, image: ImageObject):
        if not isinstance(image, ImageObject):
            raise TypeError('Image must be an ImageObject')
        self.tb_list.append(read_image(image))

    def add_document(self, document: DocumentPdf, *, apply_ocr: bool = False):
        self.tb_list.extend(
            read_document_pdf(document, document.name, pbar=self.pbar, apply_ocr=apply_ocr)
        )

    def to_data(self) -> pd.DataFrame:
        _data: list[pd.DataFrame] = []
        for m in self.tb_list:
            _data.append(m.to_data())
        if len(_data) > 0:
            return pd.concat(_data)
        return pd.DataFrame()

    def to_excel(self, file: File) -> None:
        self.to_data().to_excel(file.absolute(), index=False)


class OrganizeDocuments(object):

    def __init__(self, table_files: pd.DataFrame):
        self.table_files: pd.DataFrame = table_files
        self.pbar: ProgressBarAdapter = ProgressBarAdapter()

    def move_where_contains_text(
                self,
                find_txt: str,
                out_dir: Directory, *,
                case: bool = False,
                iqual: bool = False
            ) -> None:
        """
            Mover arquivos conforme as ocorrências de texto encontradas nos documentos.
        O arquivo é movido de diretório quando determinada ocorrência é encontrada, preservando
        o nome original.
        """
        df = self.table_files[[ColumnNames.TEXT.value, ColumnNames.FILE_PATH.value]].astype('str')
        mv_items: dict[str, list[File]] = {'src': [], 'dest': []}

        if case:
            if iqual:
                for idx, row in df.iterrows():
                    txt_in_file = f'{row[ColumnNames.TEXT.value]}'
                    if find_txt == txt_in_file:
                        src_file = File(f'{row[ColumnNames.FILE_PATH.value]}')
                        dest_file = out_dir.join_file(src_file.basename())
                        mv_items['src'].append(src_file)
                        mv_items['dest'].append(dest_file)
            else:
                for idx, row in df.iterrows():
                    txt_in_file = f'{row[ColumnNames.TEXT.value]}'
                    if find_txt in txt_in_file:
                        src_file = File(f'{row[ColumnNames.FILE_PATH.value]}')
                        dest_file = out_dir.join_file(src_file.basename())
                        mv_items['src'].append(src_file)
                        mv_items['dest'].append(dest_file)
        else:
            if iqual:
                for idx, row in df.iterrows():
                    txt_in_file = f'{row[ColumnNames.TEXT.value]}'
                    if find_txt.upper() == txt_in_file.upper():
                        src_file = File(f'{row[ColumnNames.FILE_PATH.value]}')
                        dest_file = out_dir.join_file(src_file.basename())
                        mv_items['src'].append(src_file)
                        mv_items['dest'].append(dest_file)
            else:
                for idx, row in df.iterrows():
                    txt_in_file = f'{row[ColumnNames.TEXT.value]}'
                    if find_txt.upper() in txt_in_file.upper():
                        src_file = File(f'{row[ColumnNames.FILE_PATH.value]}')
                        dest_file = out_dir.join_file(src_file.basename())
                        mv_items['src'].append(src_file)
                        mv_items['dest'].append(dest_file)
        move_list_files(mv_items)

    def move_where_math_text(
                self,
                find_txt: str,
                out_dir: Directory, *,
                separator: str = ' ',
                include_all_text: bool = False,
            ) -> None:
        """
            Mover arquivos conforme as ocorrências de texto encontradas nos documentos.
        O arquivo é movido de diretório quando determinada ocorrência é encontrada, preservando
        o nome original.
        """
        values_text = self.table_files[ColumnNames.TEXT.value].astype('str').values.tolist()
        values_files = self.table_files[ColumnNames.FILE_PATH.value].astype('str').values.tolist()
        mv_items: dict[str, list[File]] = {'src': [], 'dest': []}

        for idx, txt_in_file in enumerate(values_text):

            if find_txt.upper() in txt_in_file.upper():
                src_file = File(values_files[idx])
                # filtrar a string apagando os caracteres que antecedem o padrão informado.
                try:
                    arr = ArrayString(txt_in_file.split(separator))
                    if not include_all_text:
                        new_file_name = arr.get_next(find_txt)
                    else:
                        new_file_name = arr.get_next_all(find_txt)
                        if len(new_file_name) == 0:
                            print(f'{__class__.__name__} Falha ao obter filename')
                            continue
                        elif len(new_file_name) == 1:
                            new_file_name = new_file_name[0]
                        else:
                            new_file_name = separator.join(new_file_name)
                except Exception as e:
                    print(e)
                else:
                    if new_file_name is None:
                        print(f'{__class__.__name__} Falha, filename is None')
                        continue
                    new_file_name = fmt_str_file(new_file_name)
                    mv_items['src'].append(src_file)
                    mv_items['dest'].append(out_dir.join_file(f'{new_file_name}{src_file.extension()}'))
        move_list_files(mv_items)

    def move_where_math_column(
                self,
                df: pd.DataFrame,
                out_dir: Directory, *,
                col_find: str,
                col_new_name: str,
                cols_in_name: list[str] = [],
            ) -> None:
        """
            Mover arquivos conforme as ocorrências de texto encontradas na tabela/DataFrame df.
        o nome do novo arquivo será igual à ocorrência de texto da coluna 'col_find', podendo
        estender o nome com elementos de outras colunas, tais colunas podem ser informadas (opcionalmente)
        no parâmetro cols_in_name.
            Ex:
        Suponha que a tabela para renomear aquivos tenha a seguinte estrutura:

        A      B        C
        maça   Cidade 1 xxyyy
        banana Cidade 2 yyxxx
        mamão  Cidade 3 xyxyx

        Se passarmos os parâmetros col_find='A' e col_new_name='A' e o texto banana for
        encontrado no(s) documento, o novo nome do arquivo será banana. Caso incluir o parâmetro
        cols_in_name=['B'] o novo nome do arquivo será banana-Cidade 2 ou
        banana-Cidade 2-yyxxx (se incluir cols_in_name=['B', 'C']).

        """
        values_find = df[col_find].astype('str').values.tolist()
        values_new_name = df[col_new_name].astype('str').values.tolist()
        text_in_docs = self.table_files[ColumnNames.TEXT.value].astype('str').values.tolist()
        text_file_names = self.table_files[ColumnNames.FILE_PATH.value].astype('str').values.tolist()
        mv_items: dict[str, list[File]] = {'src': [], 'dest': []}
        if len(cols_in_name) > 0:
            cols_include_names: list[list[str]] = []
            for c in cols_in_name:
                values_include: list[str] = df[c].astype('str').values.tolist()
                cols_include_names.append(values_include)

        for num, txt in enumerate(values_find):
            for num_idx, txt_doc in enumerate(text_in_docs):
                if txt in txt_doc:
                    src_file = File(text_file_names[num_idx])
                    output_file = values_new_name[num]
                    if len(cols_in_name) > 0:
                        new = ''
                        for element in cols_include_names:
                            new = f'{new}-{element[num]}'
                        output_file = f'{output_file}-{new}'
                    output_file = fmt_str_file(output_file)
                    mv_items['src'].append(src_file)
                    mv_items['dest'].append(out_dir.join_file(f'{output_file}{src_file.extension()}'))
        move_list_files(mv_items)

